@@ -251,7 +251,7 @@ char *is_shlib(const char *dir, const char *name, int *type,
 /* update the symlink to new library */
 void link_shlib(const char *dir, const char *file, const char *so)
 {
-    int change = -1;
+    int change = 1;
     char libname[1024];
     char linkname[1024];
     struct stat libstat;
@@ -273,14 +273,29 @@ void link_shlib(const char *dir, const char *file, const char *so)
     }
 
     /* then update the link, if required */
-    if (change && !nolinks)
+    if (change > 0 && !nolinks)
     {
-	if (!lstat(linkname, &linkstat) && remove(linkname))
+	if (!lstat(linkname, &linkstat))
+	{
+	  if (!S_ISLNK(linkstat.st_mode))
+	  {
+	    warn("%s is not a symlink", linkname);
+	    change = -1;
+	  }
+	  else if (remove(linkname))
+	  {
 	    warn("can't unlink %s (%s)", linkname, strerror(errno));
-	else if (symlink(file, linkname))
+	    change = -1;
+	  }
+	}
+	if (change > 0)
+	{
+	  if (symlink(file, linkname))
+	  {
 	    warn("can't link %s to %s (%s)", linkname, file, strerror(errno));
-	else
-	    change = 1;
+	    change = -1;
+	  }
+	}
     }
 
     /* some people like to know what we're doing */
